/*	Project ir
	Infrared analyzer PIC16F628A
*/

#include <pic.h>

__CONFIG(
	  UNPROTECT
	& LVPDIS
	& BORDIS
	& MCLRDIS
	& PWRTEN
	& WDTDIS
	& INTIO
	);

__IDLOC(628A);

#define IR RB3
#define RS RA7
#define RW RA6
#define E  RB0

void lcd_w_com4(unsigned char d){
	RS = 0;
	RW = 0;
	PORTB = d << 4;
	E = 1;
	E = 0;
}

void lcd_w_chr4(unsigned char d){
	RS = 1;
	RW = 0;
	PORTB = d << 4;
	E = 1;
	E = 0;
}

void lcd_bfck(void){
	unsigned char d;
	PORTB = 0;
	TRISB7 = 1;
	RS = 0;
	RW = 1;

	do{
		E = 1;
		d = RB7;
		E = 0;
		E = 1;
		E = 0;
	} while(d);

	PORTB = 0;
	TRISB7 = 0;
}

void lcd_putcom(unsigned char d){
	lcd_bfck();
	lcd_w_com4(d >> 4);
	lcd_w_com4(d);
}

void lcd_locate(
unsigned char x, unsigned char y){
	lcd_putcom((x + 0x40 * y) | 0x80);
}

void lcd_putchr(unsigned char d){
	lcd_bfck();
	lcd_w_chr4(d >> 4);
	lcd_w_chr4(d);
}

void lcd_puts(const unsigned char *s){
	while(*s) lcd_putchr(*s++);
}

void lcd_lclr(unsigned char l){
	unsigned char i;

	lcd_locate(0, l);
	for(i = 0; i < 40; i++)
		lcd_putchr(' ');
	lcd_locate(0, l);
}

void lcd_putui(
unsigned int ui, unsigned char d){

	unsigned char i;
	unsigned char buf[5];

	for(i = 0; i < 5; i++)
		buf[i] = ' ';
	i = 4;
	do {
		buf[i] = (ui % 10) + '0';
		ui = ui / 10;
		i--;
	} while(ui > 0);

	for(i = (5 - d); i < 5; i++)
			lcd_putchr(buf[i]);
}

void ir_rdchk(void){
	TMR0 = 0;
	while(TMR0 < 80){
		if(IR) TMR0 = 0;
	}
	while(IR == 0);
}

#define MAX 64

void main(void){
	unsigned char i, n;
	unsigned char rh, rl;
	unsigned char tl, ts, tt;
	unsigned char t[MAX];

	CMCON = 0b00001010;
	VRCON = 0b10000010;

	OPTION = 0b10000101;
	PORTB = 0;
	TRISB = 0b00001000;
	PORTA = 0;
	TRISA = 0b00000000;

	TMR0 = 0; while(TMR0 < 255);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 80);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x02);

	lcd_putcom(0x28);
	lcd_putcom(0x01);
	lcd_putcom(0x0c);

	OPTION = 0b10000100;
	tl = 0;
	ts = 255;

	lcd_puts("IR analyzer");
	lcd_locate(0, 1);
	lcd_puts("Check ready");

	TMR0 = 0;
	while(TMR0 < 80)
		if(IR) TMR0 = 0;
	while(IR == 0);
	rh = TMR0;

	TMR0 = 0;
	while(IR);
	rl = TMR0;

	for(i = 0; i < MAX; i++){
		TMR0 = 0;
		while(IR == 0);
		tt = TMR0;
		TMR0 = 0;
		if(tt > tl) tl = tt;
		if(tt < ts) ts = tt;
		while(IR && (TMR0 < 80));
		t[i] = TMR0;
		if(t[i] >= 80) break;
	}
	n = i;

	lcd_lclr(0);
	lcd_puts("Catch done");
	lcd_lclr(1);
	lcd_puts("Total ");
	lcd_putui(n, 2);
	lcd_puts("bits");
	ir_rdchk();

	lcd_lclr(0);
	lcd_puts("Reader");
	lcd_lclr(1);
	lcd_puts("H="); lcd_putui(rh,3);
	lcd_puts(" L=");lcd_putui(rl,3);
	ir_rdchk();

	lcd_lclr(0);
	lcd_puts("Turn");
	lcd_lclr(1);
	lcd_puts("Max=");
	lcd_putui(tl, 3);
	lcd_puts(" Min=");
	lcd_putui(ts, 3);
	ir_rdchk();

	lcd_lclr(0);
	lcd_puts("Total ");
	lcd_putui(n, 2);
	lcd_puts("bits");

	lcd_lclr(1);
	for(i = 0; i < n; i++){
		lcd_locate(0, 1);
		lcd_puts("Bit No.");
		lcd_putui(i + 1, 2);
		lcd_putchr('=');
		lcd_putui(t[i], 3);
		ir_rdchk();
	}

	lcd_lclr(0);
	lcd_puts("IR analyzer");
	lcd_lclr(1);
	lcd_puts("Thats all.");

	while(1);
}
