/*	Project lcd877
	Liquid Crystal Display PIC16F877
*/

#include <pic.h>

__CONFIG(
	  PWRTEN
	& BORDIS
	& UNPROTECT
	& WDTDIS
	& LVPDIS
	& HS
	);

__IDLOC(F877);

#define E RB6

void lcd_w_com4(unsigned char d){
	PORTB = d & 0x0f;
	E = 1; NOP();
	E = 0;
}

void lcd_w_chr4(unsigned char d){
	PORTB = (d & 0x0f) | 0x10;
	E = 1; NOP();
	E = 0;
}

void lcd_bfck(void){
	unsigned char d;

	PORTB = 0;
	TRISB3 = 1;
	PORTB = 0b00100000;

	do{
		E = 1; NOP();
		d = RB3;
		E = 0;
		NOP(); NOP();
		E = 1; NOP();
		E = 0;
		NOP(); NOP();
	} while(d);

	PORTB = 0;
	TRISB3 = 0;
}

void lcd_putcom(unsigned char d){
	lcd_bfck();
	lcd_w_com4(d >> 4);
	lcd_w_com4(d);
}

void lcd_locate(
unsigned char x, unsigned char y){
	lcd_putcom((x + 0x40 * y) | 0x80);
}

void lcd_putchr(unsigned char d){
	lcd_bfck();
	lcd_w_chr4(d >> 4);
	lcd_w_chr4(d);
}

void lcd_puts(const unsigned char *s){
	while(*s) lcd_putchr(*s++);
}

void lcd_lclr(unsigned char l){

	unsigned char i;

	lcd_locate(0, l);
	for(i = 0; i < 40; i++)
		lcd_putchr(' ');
	lcd_locate(0, l);
}

eeprom unsigned char hc[] = {
	'0', '1', '2', '3',
	'4', '5', '6', '7',
	'8', '9', 'A', 'B',
	'C', 'D', 'E', 'F'
};

void lcd_puthc(unsigned char d){
	lcd_putchr(hc[d >> 4]);
	lcd_putchr(hc[d & 0b00001111]);
}

void lcd_putui(
unsigned int ui, unsigned char d){

	unsigned char i;
	unsigned char buf[5];

	for(i = 0; i < 5; i++)
		buf[i] = ' ';
	i = 4;
	do {
		buf[i] = (ui % 10) + '0';
		ui = ui / 10;
		i--;
	} while(ui > 0);

	for(i = (5 - d); i < 5; i++)
			lcd_putchr(buf[i]);
}

void main(void){

	OPTION = 0b10000111;
	PORTB = 0;
	TRISB = 0b0000000;

	TMR0 = 0; while(TMR0 < 255);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 80);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x02);

	lcd_putcom(0x28);
	lcd_putcom(0x01);
	lcd_putcom(0x0c);

	lcd_puts("Hello, world");
	lcd_locate(0, 1);
	lcd_puthc(12);
	lcd_putui(12, 3);
	lcd_putchr(' ');
	lcd_putui(12345, 5);

	while(1);
}
