/*	Project ssm
	Super-sonic measure PIC16F628A
*/

#include <pic.h>

__CONFIG(
	  UNPROTECT
	& LVPDIS
	& BORDIS
	& MCLRDIS
	& PWRTEN
	& WDTDIS
	& INTIO
	);

__IDLOC(628A);

#define IR RB3
#define RS RA7
#define RW RA6
#define E  RB0

void lcd_w_com4(unsigned char d){
	RS = 0;
	RW = 0;
	PORTB = d << 4;
	E = 1;
	E = 0;
}

void lcd_w_chr4(unsigned char d){
	RS = 1;
	RW = 0;
	PORTB = d << 4;
	E = 1;
	E = 0;
}

void lcd_bfck(void){

	unsigned char d;

	PORTB = 0;
	TRISB7 = 1;
	RS = 0;
	RW = 1;

	do{
		E = 1;
		d = RB7;
		E = 0;
		E = 1;
		E = 0;
	} while(d);

	PORTB = 0;
	TRISB7 = 0;
}

void lcd_putcom(unsigned char d){
	lcd_bfck();
	lcd_w_com4(d >> 4);
	lcd_w_com4(d);
}

void lcd_locate(
unsigned char x, unsigned char y){
	lcd_putcom((x + 0x40 * y) | 0x80);
}

void lcd_putchr(unsigned char d){
	lcd_bfck();
	lcd_w_chr4(d >> 4);
	lcd_w_chr4(d);
}

void lcd_puts(const unsigned char *s){
	while(*s) lcd_putchr(*s++);
}

void lcd_lclr(unsigned char l){
	unsigned char i;

	lcd_locate(0, l);
	for(i = 0; i < 40; i++)
		lcd_putchr(' ');
	lcd_locate(0, l);
}

void lcd_putui(
unsigned int ui, unsigned char d){

	unsigned char i;
	unsigned char buf[5];

	for(i = 0; i < 5; i++)
		buf[i] = ' ';
	i = 4;
	do {
		buf[i] = (ui % 10) + '0';
		ui = ui / 10;
		i--;
	} while(ui > 0);

	for(i = (5 - d); i < 5; i++)
			lcd_putchr(buf[i]);
}

unsigned char th, tl;

void interrupt entry(void){
	if(CMIF){
		CMIF = 0;
		if(C1OUT == 0){
			tl = TMR1L;
			th = TMR1H;
		}
	}
}

void main(void){

	unsigned int i;

	OPTION = 0b10000101;
	PORTB = 0;
	TRISB = 0b00000000;
	PORTA = 0;
	TRISA = 0b00001111;

	TMR0 = 0; while(TMR0 < 255);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 80);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x03);
	TMR0 = 0; while(TMR0 < 2);
	lcd_w_com4(0x02);

	lcd_putcom(0x28);
	lcd_putcom(0x01);
	lcd_putcom(0x0c);

	CMCON = 0b00001010;
	VRCON = 0b10100000 | 13; //5@5V

	INTCON = 0b11000000;
	CMIF = 0;

	T1CON = 0b00110001;

	lcd_puts("SS measure");

	while(1){

		TMR1H = 0;
		TMR1L = 0;

		for(i = 0; i < 8000; i++){
		RB2 = 1;
		NOP(); NOP(); NOP(); NOP();
		NOP(); NOP(); NOP(); NOP();
		NOP(); NOP(); NOP();
		RB2 = 0;
		}

		th = 0;
		tl = 0;

		CMIE = 1;
		while(TMR1H < 255);
		CMIE = 0;

		i = th * 256 + tl;
		lcd_locate(0, 1);

		if(i == 0)
			lcd_puts("Out of range");
		else {
			lcd_putui(i, 5);
			lcd_puts("count  ");
		}
	}
}
